package DbC;
use 5.008001;
use strict;
use warnings;

our $VERSION = "0.000001";



1;
__END__

=encoding utf-8

=head1 NAME

DbC - It's new $module

=head1 SYNOPSIS

    use DbC;

=head1 DESCRIPTION

DbC is ...

=head1 LICENSE

Copyright (C) Arun Prasaad.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

Arun Prasaad E<lt>arunbear@cpan.orgE<gt>

=cut

